export class CloudObject {
    id: string | null = null;
    className: string;
    createdAt: any;
    updatedAt: any;
    data: Record<string, any> = {};

    constructor(className: string) {
        this.className = className;
    }

    toPointer() {
        return { "__type": "Pointer", "className": this.className, "objectId": this.id };
    }

    set(json: Record<string, any>) {
        Object.keys(json).forEach(key => {
            if (["objectId", "id", "createdAt", "updatedAt", "ACL"].includes(key)) {
                return;
            }
            this.data[key] = json[key];
        });
    }

    get(key: string) {
        return this.data[key] || null;
    }

    // 修改 save 方法：根据新的接口 URL
    async save(): Promise<this> {
        let method = "POST";
        let url = `http://dev.fmode.cn:1337/parse/classes/${this.className}`;
        
        // 更新
        if (this.id) {
            url += `/${this.id}`;
            method = "PUT";
        }
        
        const body = JSON.stringify(this.data);
        const response = await fetch(url, {
            headers: {
                "accept": "*/*",
                "accept-language": "zh-CN,zh;q=0.9",
                "content-type": "text/plain;charset=UTF-8",
                "x-parse-application-id": "dev",
                "Referer": "http://127.0.0.1:4040/",
                "Referrer-Policy": "strict-origin-when-cross-origin"
            },
            body: body,
            method: method,
            mode: "cors",
            credentials: "omit"
        });

        const result = await response?.json();
        if (result?.error) {
            console.error(result.error);
        }
        if (result?.objectId) {
            this.id = result.objectId;
        }
        return this;
    }

    // 修改 destroy 方法：根据新的接口 URL
    async destroy(): Promise<boolean> {
        if (!this.id) return false;
        
        const response = await fetch(`http://dev.fmode.cn:1337/parse/classes/${this.className}/${this.id}`, {
            headers: {
                "accept": "*/*",
                "accept-language": "zh-CN,zh;q=0.9",
                "x-parse-application-id": "dev",
                "Referer": "http://127.0.0.1:4040/",
                "Referrer-Policy": "strict-origin-when-cross-origin"
            },
            body: null,
            method: "DELETE",
            mode: "cors",
            credentials: "omit"
        });

        const result = await response?.json();
        if (result) {
            this.id = null;
        }
        return true;
    }
}


export class CloudQuery {
    className: string;
    whereOptions: Record<string, any> = {};

    constructor(className: string) {
        this.className = className;
    }

    greaterThan(key: string, value: any) {
        if (!this.whereOptions[key]) this.whereOptions[key] = {};
        this.whereOptions[key]["$gt"] = value;
    }

    greaterThanAndEqualTo(key: string, value: any) {
        if (!this.whereOptions[key]) this.whereOptions[key] = {};
        this.whereOptions[key]["$gte"] = value;
    }

    lessThan(key: string, value: any) {
        if (!this.whereOptions[key]) this.whereOptions[key] = {};
        this.whereOptions[key]["$lt"] = value;
    }

    lessThanAndEqualTo(key: string, value: any) {
        if (!this.whereOptions[key]) this.whereOptions[key] = {};
        this.whereOptions[key]["$lte"] = value;
    }

    equalTo(key: string, value: any) {
        this.whereOptions[key] = value;
    }

    // 修改 get 方法：根据新的接口 URL
    async get(id: string): Promise<Record<string, any>> {
        const url = `http://dev.fmode.cn:1337/parse/classes/${this.className}/${id}?`;
        const response = await fetch(url, {
            headers: {
                "accept": "*/*",
                "accept-language": "zh-CN,zh;q=0.9",
                "x-parse-application-id": "dev",
                "Referer": "http://127.0.0.1:4040/",
                "Referrer-Policy": "strict-origin-when-cross-origin"
            },
            body: null,
            method: "GET",
            mode: "cors",
            credentials: "omit"
        });
        const json = await response?.json();
        return json || {};
    }

    // 修改 find 方法：根据新的接口 URL
    async find(): Promise<CloudObject[]> {
        let url = `http://dev.fmode.cn:1337/parse/classes/${this.className}?`;

        if (Object.keys(this.whereOptions).length) {
            const whereStr = JSON.stringify(this.whereOptions);
            url += `where=${whereStr}`;
        }

        const response = await fetch(url, {
            headers: {
                "accept": "*/*",
                "accept-language": "zh-CN,zh;q=0.9",
                "x-parse-application-id": "dev",
                "Referer": "http://127.0.0.1:4040/",
                "Referrer-Policy": "strict-origin-when-cross-origin"
            },
            body: null,
            method: "GET",
            mode: "cors",
            credentials: "omit"
        });

        const json = await response?.json();
        let list = json?.results || [];
        let objList = list.map((item: any) => this.dataToObj(item));
        return objList || [];
    }

    // 修改 first 方法：根据新的接口 URL
    async first(): Promise<CloudObject | null> {
        let url = `http://dev.fmode.cn:1337/parse/classes/${this.className}?`;

        if (Object.keys(this.whereOptions).length) {
            const whereStr = JSON.stringify(this.whereOptions);
            url += `where=${whereStr}`;
        }

        const response = await fetch(url, {
            headers: {
                "accept": "*/*",
                "accept-language": "zh-CN,zh;q=0.9",
                "x-parse-application-id": "dev",
                "Referer": "http://127.0.0.1:4040/",
                "Referrer-Policy": "strict-origin-when-cross-origin"
            },
            body: null,
            method: "GET",
            mode: "cors",
            credentials: "omit"
        });

        const json = await response?.json();
        const exists = json?.results?.[0] || null;
        if (exists) {
            let existsObject = this.dataToObj(exists);
            return existsObject;
        }
        return null;
    }

    dataToObj(exists: any): CloudObject {
        let existsObject = new CloudObject(this.className);
        existsObject.set(exists);
        existsObject.id = exists.objectId;
        existsObject.createdAt = exists.createdAt;
        existsObject.updatedAt = exists.updatedAt;
        return existsObject;
    }
}


export class CloudUser extends CloudObject {
    sessionToken: string | null = "";

    constructor() {
        super("_User");
        let userCacheStr = localStorage.getItem("NCloud/dev/User");
        if (userCacheStr) {
            let userData = JSON.parse(userCacheStr);
            this.id = userData?.objectId;
            this.sessionToken = userData?.sessionToken;
            this.data = userData;
        }
    }

    // 获取当前用户信息
    async current() {
        if (!this.sessionToken) {
            console.error("用户未登录");
            return null;
        }
        return this;
    }

    // 登录
    async login(username: string, password: string): Promise<CloudUser | null> {
        const response = await fetch(`http://dev.fmode.cn:1337/parse/login`, {
            headers: {
                "x-parse-application-id": "dev",
                "Content-Type": "application/json"
            },
            body: JSON.stringify({ username, password }),
            method: "POST"
        });

        const result = await response?.json();
        if (result?.error) {
            console.error(result?.error);
            return null;
        }
        // 登录成功后保存用户信息和 sessionToken
        this.id = result?.objectId;
        this.sessionToken = result?.sessionToken;
        this.data = result;
        
        // 将用户信息保存到 localStorage 方便下次访问
        localStorage.setItem("NCloud/dev/User", JSON.stringify(result));
        
        return this;
    }

    // 注册
    async signUp(username: string, password: string, email: string): Promise<CloudUser | null> {
        const response = await fetch(`http://dev.fmode.cn:1337/parse/users`, {
            headers: {
                "x-parse-application-id": "dev",
                "Content-Type": "application/json"
            },
            body: JSON.stringify({ username, password, email }),
            method: "POST"
        });

        const result = await response?.json();
        if (result?.error) {
            console.error(result?.error);
            return null;
        }
        // 注册成功后自动登录
        return await this.login(username, password);
    }

    // 注销
    async logOut(): Promise<void> {
        // 删除用户信息和 sessionToken
        localStorage.removeItem("NCloud/dev/User");
        this.id = null;
        this.sessionToken = null;
        this.data = {};
    }

    // 修改密码
    async changePassword(newPassword: string): Promise<boolean> {
        if (!this.sessionToken) {
            console.error("用户未登录");
            return false;
        }

        const response = await fetch(`http://dev.fmode.cn:1337/parse/users/${this.id}`, {
            headers: {
                "x-parse-application-id": "dev",
                "x-parse-session-token": this.sessionToken!,
                "Content-Type": "application/json"
            },
            body: JSON.stringify({ password: newPassword }),
            method: "PUT"
        });

        const result = await response?.json();
        if (result?.error) {
            console.error(result?.error);
            return false;
        }

        return true;
    }

    // 修改用户信息
    async updateUserInfo(newData: Record<string, any>): Promise<boolean> {
        if (!this.sessionToken) {
            console.error("用户未登录");
            return false;
        }

        const response = await fetch(`http://dev.fmode.cn:1337/parse/users/${this.id}`, {
            headers: {
                "x-parse-application-id": "dev",
                "x-parse-session-token": this.sessionToken!,
                "Content-Type": "application/json"
            },
            body: JSON.stringify(newData),
            method: "PUT"
        });

        const result = await response?.json();
        if (result?.error) {
            console.error(result?.error);
            return false;
        }

        // 更新本地缓存
        this.data = { ...this.data, ...newData };
        localStorage.setItem("NCloud/dev/User", JSON.stringify(this.data));

        return true;
    }
}

           
